@@ -10,9 +10,10 @@ import gsm0338
 # pylint: disable=import-error,no-name-in-module
 from construct.lib.containers import Container, ListContainer
 from construct.core import EnumIntegerString
-from construct import Adapter, Prefixed, Int8ub, GreedyBytes, Default, Flag, Byte, Construct, Enum
-from construct import BitsInteger, BitStruct, Bytes, StreamError, stream_read_entire, stream_write
+from construct import Adapter, Prefixed, Int8ub, Default, Flag, Byte, Construct, Enum
+from construct import BitsInteger, BitStruct, StreamError, stream_read_entire, stream_write
 from construct import SizeofError, IntegerError, swapbytes
+import construct
 from construct.core import evaluate
 from construct.lib import integertypes
 
@@ -33,6 +34,19 @@ from osmocom.utils import b2h, h2b, swap_nibbles, int_bytes_required
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+class Bytes(construct.Bytes):
+    """Just like construct.Bytes but supporting hex-string input."""
+    def _build(self, obj, stream, context, path):
+        data = h2b(obj) if isinstance(obj, str) else obj
+        return super()._build(data, stream, context, path)
+
+@construct.core.singleton
+class GreedyBytes(construct.GreedyBytes.__class__):
+    """Just like construct.GreedyBytes but supporting hex-string input."""
+    def _build(self, obj, stream, context, path):
+        data = h2b(obj) if isinstance(obj, str) else obj
+        return super()._build(data, stream, context, path)
+
 
 class HexAdapter(Adapter):
     """convert a bytes() type to a string of hex nibbles."""
